/* PressureSensor_hw
   Basic example of using I2C-based template library for Bosch BMP085 digital pressure sensor.
   Created by Adrian Studer, April 2013.
   Distributed under MIT License, see license.txt for details.   
*/

#include "Wire.h"          // required by BMP085 library
#include "BMP085_t.h"      // import BMP085 template library

BMP085<3> PSensor;         // instantiate sensor, 0..3 = precision of pressure reading

void setup()
{
  Serial.begin(9600);      // initialize serial, used to display readings in Serial Monitor
  Wire.begin();            // initialize I2C that connects to sensor
  PSensor.begin();        // initalize pressure sensor
  Serial.println("Pressure sensor demo with hardware I2C support");   
}

void loop()
{
  PSensor.refresh();                     // read current sensor data
  PSensor.calculate();                   // run calculations for temperature and pressure
  float t = PSensor.temperature;         // temperature in 0.1 Celsius units 
  float p = PSensor.pressure;            // pressure in Pascal units
  Serial.print("Temperature: ");         
  Serial.print(t/10,1);                  // display temperature in Celsius (1 decimal)
  Serial.print(" C ");
  Serial.print("Pressure: ");
  Serial.print((p+1440)/100,1);          // display pressure in hPa (with correction for altitude of Debrecen)
  Serial.println(" hPa");
  delay(5000);                           // wait a while before next loop
}
 

