/* PressureSensor_sw
 * Basic example to demonstrate the capanilities of the
 * Bosch BMP085/BMP180 pressure sensors.
 * In this project sotware I2C communication was used by using
 * the Master SoftWire library of Rei Vilo.
 * Link: http://embeddedcomputing.weebly.com/master-isup2c-software-library.html
 * The data from the pressure sensor is read out by the help of the
 * the BMP085 template library created by Adrian Studer.
 * Link: https://github.com/astuder/BMP085-template-library-Energia
 *
 * Hardware connections:
 * MSP430G2553 Launchpad    BMP085 breakout module (mine has on board voltage regulator!)
 *   TP1 (+5V)               VIN
 *   GND (pin20)             GND
 *   P2_0 (pin8)             SCL
 *   P2_1 (pin9)             SDA   
 */
#include "SoftwareWire.h"  // required by BMP085 library
#define SCL_PIN P2_0       //I2C bus clock line
#define SDA_PIN P2_1       //I2C bus data line
SoftwareWire Wire(SDA_PIN, SCL_PIN); //Instantiate SW I2C channel
#include "BMP085_t.h"     // import BMP085 template library
BMP085<3> PSensor;         // instantiate sensor, 0..3 = precision of pressure reading

void setup()
{ delay(1000);
  Serial.begin(9600);      // initialize serial, used to display readings in Serial Monitor
  Wire.begin();           // initialize I2C that connects to sensor
  PSensor.begin();         // initalize pressure sensor
  Serial.println("Pressure sensor demo with software I2C");
}

void loop()
{
  PSensor.refresh();                     // read current sensor data
  PSensor.calculate();                   // run calculations for temperature and pressure
  float t = PSensor.temperature;         // temperature in 0.1 Celsius units 
  float p = PSensor.pressure;            // pressure in Pascal units
  Serial.print("Temperature: ");         
  Serial.print(t/10,1);                  // display temperature in Celsius (1 decimal)
  Serial.print(" C ");
  Serial.print("Pressure: ");
  Serial.print((p+1440)/100,1);          // display pressure in hPa (with correction for altitude of Debrecen)
  Serial.println(" hPa");
  delay(5000);                           // wait a while before next loop
}

